<?php
/**
 * Created by PhpStorm.
 * User: crose
 * Date: 2/2/20
 * Time: 9:02 AM
 */

namespace IMATHUZH\Qfq\Core\Typo3;

use IMATHUZH\Qfq\Core\Helper\Path;
use IMATHUZH\Qfq\Core\Helper\Support;


/**
 * Class T3Handler
 * @package IMATHUZH\Qfq\Core\Typo3
 */
class T3Handler {

    /**
     * @var
     */
    private static $objectManager = null;

    /**
     * Call logoff of current FE User
     */
    public static function feLogOff() {

        // Restore T3 ErrorHandler. T3 throws exceptions - those should be handled by T3!
        restore_error_handler();

        $GLOBALS['TSFE']->fe_user->logoff();

        // Activate QFQ ErrorHandler again.
        Support::setQfqErrorHandler();
    }


    /**
     * Convert a cleartext password to a hash. Respects if 'salted passwords' are enabled.
     *
     * @param string $newPassword
     * @return string
     * @throws \UserFormException
     */
    public static function getHash($newPassword) {

        // Restore T3 ErrorHandler. T3 throws exceptions - those should be handled by T3!
        restore_error_handler();

        $saltedPassword = null;
        self::t3AutoloadIfNotRunning();

        if (class_exists('\TYPO3\CMS\Core\Utility\GeneralUtility')
            && class_exists('\TYPO3\CMS\Core\Crypto\PasswordHashing\PasswordHashFactory')) {

            // Typo3 version >=9
            $hashInstance = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance(\TYPO3\CMS\Core\Crypto\PasswordHashing\PasswordHashFactory::class)
                ->getDefaultHashInstance('FE');
            $saltedPassword = $hashInstance->getHashedPassword($newPassword);

        } elseif (class_exists('\TYPO3\CMS\Saltedpasswords\Utility\SaltedPasswordsUtility') && class_exists('\TYPO3\CMS\Saltedpasswords\Salt\SaltFactory')) {

            // Typo3 version <=8
            // Legacy code based on https://docs.typo3.org/typo3cms/extensions/saltedpasswords/8.7/DevelopersGuide/Index.html
            if (\TYPO3\CMS\Saltedpasswords\Utility\SaltedPasswordsUtility::isUsageEnabled('FE')) {
                $objSalt = \TYPO3\CMS\Saltedpasswords\Salt\SaltFactory::getSaltingInstance(NULL);
                if (is_object($objSalt)) {
                    $saltedPassword = $objSalt->getHashedPassword($newPassword);
                }
            }

        }

        // Activate QFQ ErrorHandler again.
        Support::setQfqErrorHandler();

        if (is_null($saltedPassword)) {
            // Fallback, use Argon2i algorithm (with current default options of typo3 in year 2020)
            $saltedPassword = password_hash($newPassword, PASSWORD_ARGON2I, ['memory_cost' => 65536, 'time_cost' => 16, 'threads' => 1]);
        }

        return $saltedPassword;
    }

    /**
     * Check if the salted password corresponds to the password.
     *
     * @param string $saltedPassword
     * @param string $password
     * @return bool
     * @throws \UserFormException
     */
    public static function checkPassword($saltedPassword, $password) {

        $success = null;
        self::t3AutoloadIfNotRunning();

        // Restore T3 ErrorHandler. T3 throws exceptions - those should be handled by T3!
        restore_error_handler();

        if (class_exists('\TYPO3\CMS\Core\Utility\GeneralUtility')
            && class_exists('\TYPO3\CMS\Core\Crypto\PasswordHashing\PasswordHashFactory')) {

            // Typo3 version >=9
            $success = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance(\TYPO3\CMS\Core\Crypto\PasswordHashing\PasswordHashFactory::class)
                ->get($saltedPassword, 'FE') # or getDefaultHashInstance($mode)
                ->checkPassword($password, $saltedPassword);

        } elseif (class_exists('\TYPO3\CMS\Saltedpasswords\Utility\SaltedPasswordsUtility') && class_exists('\TYPO3\CMS\Saltedpasswords\Salt\SaltFactory')) {

            // Typo3 version <=8
            // Legacy code based on https://docs.typo3.org/typo3cms/extensions/saltedpasswords/8.7/DevelopersGuide/Index.html
            if (\TYPO3\CMS\Saltedpasswords\Utility\SaltedPasswordsUtility::isUsageEnabled('FE')) {
                $objSalt2 = \TYPO3\CMS\Saltedpasswords\Salt\SaltFactory::getSaltingInstance($saltedPassword);
                if (is_object($objSalt2)) {
                    $success = $objSalt2->checkPassword($password, $saltedPassword);
                }
            }

        }

        // Activate QFQ ErrorHandler again.
        Support::setQfqErrorHandler();

        if (is_null($success)) {
            // fallback use php function directly
            $success = password_verify($password, $saltedPassword);
        }

        return $success;
    }

    /**
     * Load Typo3 autoloader if Typo3 is not instantiated
     *
     * @throws \UserFormException
     */
    public static function t3AutoloadIfNotRunning() {

        if (!class_exists('\TYPO3\CMS\Core\Utility\GeneralUtility')) {

            if (file_exists(Path::absoluteApp('vendor/autoload.php'))) {
                // Typo3 version >=9
                require Path::absoluteApp('vendor/autoload.php');
            } else {
                // Typo3 version <=8
                require Path::absoluteApp('typo3_src/vendor/autoload.php');
            }

            // run typo3 bootstrap if not yet happened. Necessary if run in unittest.
            if (!defined('TYPO3_MODE')) {

                // NOTE: Deprecated: Deprecation: #85821 - bootstrap methods
                \TYPO3\CMS\Core\Core\Bootstrap::getInstance()
                    ->setRequestType(TYPO3_REQUESTTYPE_AJAX)
                    ->baseSetup(0);
//                Alternate error fix if you don't want to run Typo3 bootstrap:
//                error_reporting(E_ALL & ~(E_STRICT | E_NOTICE | E_DEPRECATED));
                define('TYPO3_MODE', 'FE');
            }
        }
    }

    /**
     * Update a single key/value pair in `typo3conf/LocalConfiguration.php` QFQ config.
     *
     * @param $key
     * @param $value
     * @throws \UserFormException
     */
    public static function updateT3QfqConfig($key, $value) {

        if (defined('PHPUNIT_QFQ')) {
            // There is no  typo3conf/LocalConfiguration.php in phpunit.
            return;
        }

        // Restore T3 ErrorHandler. T3 throws exceptions - those should be handled by T3!
        restore_error_handler();

        self::t3AutoloadIfNotRunning();

        $configurationManager = \TYPO3\CMS\Core\Utility\GeneralUtility::makeInstance('TYPO3\\CMS\\Core\\Configuration\\ConfigurationManager');

        // Same as $GLOBALS['TYPO3_CONF_VARS']['EXT']['extConf']['qfq']
        $configT3 = $configurationManager->getLocalConfiguration();
        if (isset($configT3['EXTENSIONS'][EXT_KEY])) {
            // Typo3 version >=9
            $configQfq = $configT3['EXTENSIONS'][EXT_KEY];
        } else {
            // Typo3 version <=8
            $configQfq = unserialize($configT3['EXT']['extConf'][EXT_KEY]);
        }

        // Set new value
        $configQfq[$key] = $value;

        // Prepare
        if (isset($configT3['EXTENSIONS'][EXT_KEY])) {
            // Typo3 version >=9
            $configT3['EXTENSIONS'][EXT_KEY] = $configQfq;
        } else {
            // Typo3 version <=8
            $configT3['EXT']['extConf'][EXT_KEY] = serialize($configQfq);
        }

        // Write new config to typo3conf/LocalConfiguration.php
        $configurationManager->writeLocalConfiguration($configT3);

        // Activate QFQ ErrorHandler again.
        Support::setQfqErrorHandler();
    }
}